Scriptname Bo57:Bo57_CL_main_playerRefScript extends ReferenceAlias

; Properties
Group ScriptLibsAndTools
  DD:DD_Library                 Property libs                      Auto Const ; DD Base API
  Bo57:Bo57_MCMandToolsScript   Property Tools                     Auto Const ; Some common functions
  WorkshopParentScript          Property WorkshopParent            Auto Const
endGroup

Group ActorsAndActorValues
  Actor                Property Player                              Auto Const
  actorvalue           Property Paralyze                            Auto
  actorvalue           Property lockpicking                         Auto Const
endGroup

Group Miscobjects
  Miscobject           Property BobbyPin                            Auto Const
  Miscobject           Property SynthComponent                      Auto Const
endGroup

Group MiscProperties
  Message              Property Bo57_Ask_LockSafe                   Auto Const Mandatory
  RefCollectionAlias   Property Bo57_SafesforKeysRC                 Auto Const ; Const or not ??
endGroup


Group TrapTriggerEffects
  Explosion            Property Bo57_Unstable_GasExplosion          Auto Const        ; For Safe
  Explosion            Property Bo57_GasExplosion_Regular           Auto Const        ; Normal
  Explosion            Property Bo57_GasExplosion_2                 Auto Const        ; like unstable, with small radius

  ImageSpaceModifier   Property FadeToBlackHold                     Auto Const
  spell                Property Bo57_CryoParalyze                   Auto Const
  objectreference      Property Bo57_PositionmarkRef                Auto Const
endGroup

Group Globals
  GlobalVariable       Property Bo57_CL_slaveitemsdropchance        Auto     ; for items from slaveitems list
  GlobalVariable       Property Bo57_CL_CursedLootChance            Auto     ; for items from other list
  GlobalVariable       Property Bo57_CL_CursedCorpseChance          Auto     ; for items from corpses
  GlobalVariable       Property Bo57_CL_CursedRobotMult             Auto     ; for items from corpse robots
  GlobalVariable       Property Bo57_CL_CursedNakedMult             Auto     ; for items from corpse robots
  GlobalVariable       Property Bo57_NudePantAndTop                 Auto     ; Nude must be both pants and top to count
  GlobalVariable       Property Bo57_ShowRollAndChance              Auto     ; show the roll and chance

  GlobalVariable       Property Bo57_CL_KeyLoseChance               Auto
  GlobalVariable       Property Bo57_CL_KeyDropChance               Auto
  GlobalVariable       Property Bo57_CL_Keysonlywhenrestrained      Auto
endGroup

Group Keywords
  Keyword              Property LinkTerminalSafe                    Auto Const  Mandatory
  Keyword              Property WorkshopcontainerKeyword            Auto Const  Mandatory
  Keyword              Property DD_kw_ItemType_WristCuffs           Auto Const  Mandatory
  Keyword              Property LocTypeWorkshopSettlement           Auto Const  Mandatory
endGroup

Group FormLists
  FormList             Property Bo57_CL_containerlist               Auto Const
  FormList             Property Bo57_CL_slaveitemlist               Auto Const
  FormList             Property Bo57_CL_otheritemlist               Auto Const
  FormList             Property Bo57_CL_parasiteitemlist            Auto Const
endGroup

Group SAKR
  FormList             Property SAKR_alltops                        Auto Const
  FormList             Property SAKR_allbottoms                     Auto Const
  FormList             Property SAKR_allskirts                      Auto Const
  FormList             Property SAKR_alllegs                        Auto Const
  FormList             Property SAKR_alllewd                        Auto Const
endGroup

Group RaceLists
  FormList             Property Bo57_RobotRaces                     Auto Const
  FormList             Property Bo57_AnimalRaces                    Auto Const
  FormList             Property Bo57_HumanoidRaces                  Auto Const
endGroup

;--------------------------------------------------------------------
; Internal Variables

int               iScriptVersion

formlist          kEquipFromList
bool              bUseParasiteList
bool              bUseSafeList

Int               iLastLootedCounter
Int               iLastLootedMaxSize = 12
int[]             iaLastLooted

objectreference   kLastSafeTried
objectreference   kLastSafeUsed

int               iNumberOfItems
int               itemsequipped

bool              bTryStripBody

Keyword[]         AAF_protectedKeywords

;--------------------------------------------------------------------
bool function hasbeenlooted (int containerID)

   return (iaLastLooted.find (containerID) >= 0)

endfunction

function rememberID (int containerID)

    iLastLootedCounter += 1

    if iLastLootedCounter >= iLastLootedMaxSize
        iLastLootedCounter = 0
    endIf

    iaLastLooted[iLastLootedCounter] = containerID

endfunction

;---------------------------------------------------------------
;
;---------------------------------------------------------------

function Log (string s)
   Debug.Trace ("[Bo57] " + s)
Endfunction

;---------------------------------------------------------------
;
;---------------------------------------------------------------

int function getscriptversion ()
    return 173
endfunction


;--------------------------------------------------------------------

; Some things are done in onplayerloadgame and again when the quest restarts,
; Double calling - for compatibility - better try again to prevent this

Event OnPlayerLoadGame()
    log ("OnPlayerLoadGame")

    kLastSafeUsed = None
    Gotostate ("")

    RegisterForExternalEvent ("OnMCMOpen", "OnMCMOpen");

    if iScriptVersion != getscriptversion ()
      Debug.messagebox ("Restart of cursed loot quest in MCM required")
      getOwningQuest().Stop()
    endif

EndEvent

function CursedQuestInit ()
    log ("CursedQuestInit")

    RemoveAllInventoryEventFilters()
    AddInventoryEventFilter(None)
    iaLastLooted = new int[12]

    RegisterForExternalEvent ("OnMCMOpen", "OnMCMOpen");
    RegisterForMenuOpenCloseEvent ("LockpickingMenu")
    RegisterForMenuOpenCloseEvent ("ContainerMenu")
    RegisterForMenuOpenCloseEvent ("WorkshopMenu")
    RegisterForMenuOpenCloseEvent ("PipBoyMenu")

    iScriptVersion = getscriptversion ()

endfunction

;---------------------------------------------------------------
;
;---------------------------------------------------------------

Event OnAliasInit()
   if Player.getactorbase ().getsex () < 1
      Debug.messagebox ("Currently only for females")
      Getowningquest().stop ()
   else
      CursedQuestInit ()
      OnPlayerLoadGame()
   endif

   kLastSafeUsed = None
   Gotostate ("")
endEvent


Event OnAliasShutdown()
   log ("Cursed quest had stopped")

   RemoveAllInventoryEventFilters()

   ; kLastSafeUsed and kLastSafeTried already cleaned

   objectreference safe
   int i = Bo57_SafesforKeysRC.GetCount()

   if i
      while i
         i -= 1
         safe = Bo57_SafesforKeysRC.getat (i)
         Log ("cleaning safe " + safe)

         if safe
            safe.RemoveItem (libs.DD_kw_IsDDKey, -1, true, Player)
         endif
      endwhile

      Tools.convertDDkeys (false)           ; convert Keys back for Player with notification
   endIf
endEvent

;---------------------------------------------------------------
; MCM Events
;---------------------------------------------------------------

Function OnMCMOpen()
   ; Debug.Notification ("Devious device menu OPENED")   ; Test

   quest CLQ = self.Getowningquest () as Quest;
   Bo57:Bo57_CursedLootControllerScr CLQ_Script = CLQ as Bo57:Bo57_CursedLootControllerScr

   CLQ_Script.bOptMCM_Restrained                = Tools.isHeavyRestrained ( Player )
   CLQ_Script.bOptMCM_NotRestrained             = !CLQ_Script.bOptMCM_Restrained

   MCM.RefreshMenu()
EndFunction

;---------------------------------------------------------------
;
;---------------------------------------------------------------

Event OnMenuOpenCloseEvent (string asMenuName, bool abOpening)

  ; log ("Menu open/closed " + asMenuName)
  If asMenuName == "Workshopmenu"
    WorkshopMode (abOpening)
  elseif (abOpening)
    if (asMenuName == "PipBoyMenu")
      Tools.convertDDkeys (false)               ; only converts cursed keys in players inventory back to regular dd keys
    else
      If Bo57_CL_slaveitemsdropchance.GetValue()
        objectreference cont =  LL_FourPlay.LastCrossHairRef()
        if Cont
          if (asMenuName == "LockpickingMenu")

            if Cont.GetBaseObject().HasKeyword(LinkTerminalSafe)
              kLastSafeTried = Cont

              if Player.IsInPowerArmor()
                Debug.Notification ("I can't feel anything in this power armor.")
                Player.SwitchToPowerArmor (None)
              endIf

              GotoState ("TryOpenSafe")
            endif
          elseif asMenuName == "ContainerMenu"

            if Cont.GetBaseObject().HasKeyword(LinkTerminalSafe)
              kLastSafeUsed = Cont
              GotoState ("UnlockedSafeOpened")
            endif
          endif
        endif
      endif
    endif
  else
    log ("Menu close event " + asMenuName)
  endif

endEvent


;---------------------------------------------------------------
;
;---------------------------------------------------------------

; Event OnWorkshopMode(bool aStart)
function WorkshopMode (bool aStart)
   ; If the Quest is not running and the playeralias not filled, we do not need to care
   ; because all cursed keys previously in safes are already returned to the Player

   if ( aStart )
   else
      ; log ("Workshop Mode Ended, now get the current workshop")

      objectreference ws_maincontainer = WorkshopParent.CurrentWorkshop.getreference ()
      if ws_maincontainer
         if ws_maincontainer.getitemcount (libs.DD_kw_IsDDKey)
            ws_maincontainer.removeitem (libs.DD_kw_IsDDKey, -1)
            Debug.Messagebox ("Your cursed keys has destroyed themselves, after the safe they were stored in was treated badly.")

            objectreference safe
            int i = Bo57_SafesforKeysRC.getcount ()

            log ("now clean up unused safes in the refcollection " + i)

            while i
               i -= 1
               safe = Bo57_SafesforKeysRC.getat (i)

               if safe
                  log ("disabled " + safe.isdisabled () + " destroyed " + safe.isdestroyed ())
               endif
            endwhile

         endif
      endif
   endif

Endfunction


;---------------------------------------------------------------
;
;---------------------------------------------------------------

; true if the Player seems to own that container
Bool Function isownedbox (ObjectReference boxref)

    ; ownershiphandling is a horror without usable support in the condition functions

    if boxref.iscreated()               ; I assume this are all placed containers in settlements

        return true
    else
        Bool isOwnedWorkshoploc
        Location testloc = Player.GetCurrentLocation()
        if testloc.haskeyword (LocTypeWorkshopSettlement)
            WorkshopScript workshopRef = workshopParent.GetWorkshopFromLocation (testloc)

            if workshopRef && workshopRef.OwnedByPlayer

                ; we are in a settlement with owned workshop, bed should be usable
                ; Test if we own the workshop is necessary
                isOwnedWorkshoploc = True
            endif
        endif

        if isOwnedWorkshoploc

            if BoxREF.hasowner ()
                if Player.Getactorbase() == BoxREF.getactorowner()
                   return true
                endif
                ; cell ownership no need to check
                ; log ("boxref owner: " + BoxREF.getactorowner())
                ; log ("boxref refowner: " + BoxREF.GetActorRefOwner())
                ; log ("boxref faction: " + BoxREF.getfactionowner())

            else
                log ("boxref no owner ")

                return true        ; indirect ownership in workshop
            endif
        endif
    endif

    return false                ; box considered as can trigger

endFunction


;--------------------------------------------------------------------

int Function IsValidContainer (ObjectReference box)
    If (box As Actor) && (box As Actor).IsDead()
        return 1
    EndIf

    If Bo57_CL_containerlist.HasForm(box.GetBaseObject()) && !isownedbox( box )
        return 2
    EndIf

    return 0
EndFunction

;--------------------------------------------------------------------
; calculate if trap should trigger
;--------------------------------------------------------------------

bool function triggerTrap ( float basechance )

    if Tools.isNaked ( Player )
        basechance *= Bo57_CL_CursedNakedMult.GetValue() as float
    endIf

    Float roll = Utility.RandomFloat(0.0, 100.0)

    if Bo57_ShowRollAndChance.GetValueInt() == 1
      Debug.Notification("[Bo57] Roll " + (roll as String) + " &lt;= " + (basechance as String) )
    endIf

    Return roll <= basechance
endfunction


;*******************************************************************
;Fade Functions

Function DoFadeOut (Float Time)
    FadeToBlackHold.ApplyCrossFade (Time)
    Utility.Wait (Time)
    Game.FadeOutGame (False,True,50,1)
EndFunction


Function DoFadeIn_plus (Float Time)
    utility.wait (2)

    Game.FadeOutGame (False,  True,0.1, 4.0)
    ImageSpaceModifier.RemoveCrossFade (4.0)

    ; Now wait a little and reenable the Player
    if itemsequipped
        if itemsequipped > 1
            libs.notify ("Some restraints locked itself on you !!!")
        else
            libs.notify ("A restraint locked itself on you !!!")
        endif
    endif

    Utility.Wait (time)                ; Wait some more
    Player.SetValue (Paralyze, 0)
    Tools.paralyzeFollowers ()   ; free followers
EndFunction

;--------------------------------------------------------------------
;
;--------------------------------------------------------------------

function placeExplosionAndWait (float x, float y, float sec)

    Bo57_PositionmarkRef.MoveTo (Player, x, y)
    Bo57_PositionmarkRef.placeatme (Bo57_GasExplosion_2, 1)
    if sec > 0.0
        Utility.wait (sec)
    endif

endfunction

function DropWeapon ()
    if Player.isweapondrawn ()
        weapon wpn = Player.getequippedweapon ()
        if wpn
            Player.dropobject (wpn)
        endif
    endif
endfunction

;--------------------------------------------------------------------
;
;--------------------------------------------------------------------

function startDeviousEffect ()

    if UI.IsMenuOpen ("ContainerMenu")
        UI.CloseMenu ("ContainerMenu")
     endif

    placeExplosionAndWait (-120, -120, 0.07)
    placeExplosionAndWait (-120,  120, 0.07)
    placeExplosionAndWait ( 120,  120, 0.07)

    Bo57_PositionmarkRef.MoveTo (Player, 120, -120)
    Bo57_PositionmarkRef.placeatme (Bo57_Unstable_GasExplosion, 1)
;   placeExplosionAndWait ( 120, -120, 0.0)

    Tools.identifyPossibleFollowers (Bo57_GasExplosion_Regular)

    Tools.paralyzeFollowers (1)

    If Player.IsInPowerArmor ()
        iNumberOfItems = 0
    else
        Player.PushActorAway (Player, 2.0)

        DropWeapon ()

        Player.SetValue (Paralyze, 1)

        iNumberOfItems = 1
    endif

    DoFadeOut (4.0)

    ; debug.notification ("now fade out")

    itemsequipped = 0
    StartTimer(1.0)                ; Middle of the black phase ?

    ; debug.notification ("now fade in")

    DoFadeIn_Plus (3.0)

endfunction

;--------------------------------------------------------------------
;
;--------------------------------------------------------------------

function startDeviousEffectSafe ()

    Player.placeatme (Bo57_Unstable_GasExplosion, 3)

    If Player.hasspell (Bo57_CryoParalyze) == false
        Player.addspell (Bo57_CryoParalyze, false)
    endif

    Bo57_CryoParalyze.Cast(Player, Player)
    DropWeapon ()

    Player.SetValue (Paralyze, 1)          ; paralyze before

    DoFadeOut (4.0)
    Player.SetValue (Paralyze, 1)

    iNumberOfItems = Math.Ceiling((kLastSafeTried.Getlocklevel () + 12) / 25) + 1
    itemsequipped = 0

    ; log ("number of items " + iNumberOfItems)
    StartTimer (1.0, 1)                ; Middle of the black phase
    utility.wait (2)
    Bo57_CryoParalyze.Cast(Player, Player)

    DoFadeIn_Plus (8.0)

endfunction


;--------------------------------------------------------------------
;
;--------------------------------------------------------------------

function addkeytocontainer (ObjectReference akSourceContainer, Bool fromSafe = False)

    ; If nothing else happened: Check for generic cursed loot:

    Bool bIsRetrained = Player.WornHasKeyword(libs.DD_kw_RenderedItem)

    If Bo57_CL_Keysonlywhenrestrained.getvalue ()
        if !bIsRetrained
            RETURN
        endif
    endif

    Bool IsHeavilyRestrained = Tools.isHeavyRestrained( Player )

    Float KeyChance = Bo57_CL_KeyDropChance.GetValue()
    If fromSafe
        KeyChance *= 2
    endIf

    If bIsRetrained
        KeyChance *= 2       ; give the Player a higher chance to find keys if she is wearing a restraint.


        If IsHeavilyRestrained
            ; give the Player a much higher chance to find keys if she is wearing debilitating restraints.
            KeyChance *= 2
        EndIf
    EndIf

    If Utility.RandomFloat(0.0, 99.9) < KeyChance
        If Utility.RandomInt (1,2) == 1 || IsHeavilyRestrained
            akSourceContainer.additem(libs.DD_RestraintsKey, 1, true)
        Else
            akSourceContainer.additem(libs.DD_ChastityKey, 1, true)
        EndIf
    EndIf

endfunction

;--------------------------------------------------------------------
;
;--------------------------------------------------------------------
Event OnItemAdded (Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
    if !akSourceContainer || !akBaseItem || akSourceContainer == Tools.Bo57_FoodLocker || hasbeenlooted(akSourceContainer.GetFormID())
        return
    endif

    rememberID (akSourceContainer.GetFormID())               ; blocks further actions immediate

    if kLastSafeTried == aksourcecontainer
        ; Check if it comes from a safe or locked container

        if Bo57_CL_slaveitemsdropchance || kLastSafeTried.getitemcount (libs.DD_kw_IsDDKey)
             ; Only add keys to safes, if there was also a chance to trigger a trap
            addkeytocontainer (akSourceContainer, True)
        endif

        kLastSafeTried = None                  ; Only one times key chance, independant of cursed keys
        Return
    endif

    int contype = IsValidContainer(akSourceContainer)  ; 1 = corpse, 2 = container, 0 = notvalid

    ; Belted Quest is running or the Player is still in the vault or is otherwise busy:
    If (contype == 0) || libs.PlayerIsBusy (IncludePowerArmor = false) ; libs.PlayerIsBusy (IncludePowerArmor = true)
        Return
    EndIf

    Race kContainerRace = (akSourceContainer As Actor).GetLeveledActorBase().GetRace()
    bool bTrapfired

    if contype == 1
        Float fCorpseChance = Bo57_CL_CursedCorpseChance.GetValue()

        if Bo57_RobotRaces.Find( kContainerRace ) >= 0
          fCorpseChance *= Bo57_CL_CursedRobotMult.GetValue()
        endIf

        if fCorpseChance > 100.0
          fCorpseChance = 100.0
        endIf

        bTrapfired = triggerTrap ( fCorpseChance )
    else
        bTrapfired = triggerTrap ( Bo57_CL_CursedLootChance.GetValue() )
    endif

    If bTrapfired
        bUseParasiteList = false
        bUseSafeList     = false

        if contype == 1
          if Bo57_HumanoidRaces.Find( kContainerRace ) >= 0
            akSourceContainer.additem(SynthComponent, 1, true)
          elseIf Bo57_AnimalRaces.Find( kContainerRace ) >= 0
            bUseParasiteList = true
          endIf
        endIf

        startDeviousEffect ()

        return
    EndIf

EndEvent

;--------------------------------------------------------------------
;
;--------------------------------------------------------------------

Event OnTimer(int aiTimerID)
    bool bRemoveKeys

    if iNumberOfItems > 4
        iNumberOfItems = 4
    elseIf iNumberOfItems < 1 || bUseParasiteList
        iNumberOfItems = 1
    endIf

    int i = iNumberOfItems

    if bUseSafeList
        kEquipFromList = Bo57_CL_slaveitemlist
        bRemoveKeys    = true          ; Always loose restraintkeys
    else
        if bUseParasiteList
            kEquipFromList = Bo57_CL_parasiteitemlist
        elseIf Tools.RoggvirGetDDList()
            kEquipFromList = Tools.Bo57_CL_rogglist
        else
            kEquipFromList = Bo57_CL_otheritemlist
        endif

        If Utility.RandomFloat(0.0, 99.9) < Bo57_CL_KeyLoseChance.GetValue()
            bRemoveKeys = true
        endif
    endif

    bool bTrapPlayer = Tools.equipFollowerItems(kEquipFromList, bRemoveKeys)

    If Player.IsInPowerArmor ()
        Player.SwitchToPowerArmor (None)
        bTrapPlayer = false
    endif

    if bTrapPlayer
        bTryStripBody = True
        RegisterForAnimationEvent(Player,"BasePoseEnter")         ;
        ; RegisterForAnimationEvent(Player,"FootRight")             ; Fires
        ; RegisterForAnimationEvent(Player,"FootLeft")              ; fires


        if aiTimerID == 1 && !bUseParasiteList
            i += Tools.TryToAddMoreItems (0.6)
        endif

        while i > 0
            i -= 1;
            If libs.EquipRandomDevice(Player, kEquipFromList, 1)
                itemsequipped += 1
            else
                i = 0;          Terminate without further trials to equip something
            endif
        Endwhile

        If bRemoveKeys
            Player.RemoveItem(libs.DD_RestraintsKey, -1)
            Player.RemoveItem(libs.DD_ChastityKey, -1)
        EndIf
    endif

    bUseParasiteList = false
    bUseSafeList     = false
EndEvent


Event OnAnimationEvent(ObjectReference akSource, string asEventName)

    If akSource == Player

        log ("Animation " + asEventName)

        if Player.wornhaskeyword (DD_kw_ItemType_WristCuffs) || Player.wornhaskeyword (libs.DD_kw_EffectType_BondageMittens)
            Tools.stripActor( Player )

            if Tools.isWeaponIdleFixed ( Player )

                ; UnRegisterForAnimationEvent(Player,"FootRight")             ; Fires
                ; UnRegisterForAnimationEvent(Player,"FootLeft")              ; fires

                UnRegisterForAnimationEvent(Player,"BasePoseEnter")
            endif
        else
            UnRegisterForAnimationEvent(Player,"BasePoseEnter")
        endif
    EndIf

EndEvent
;---------------------------------------------------------------
;
;---------------------------------------------------------------

state TryOpenSafe

    ;------------------------------------------------------------
    ;
    Event OnBeginState (string asOldState)

        RemoveInventoryEventFilter (None)
        AddInventoryEventFilter (BobbyPin)

        ; log ("TryOpensafe begin " + kLastSafeTried)

    endEvent

    ;------------------------------------------------------------
    ;
    Event OnEndState (string asOldState)

        log ("TryOpensafe end")
        RemoveInventoryEventFilter (BobbyPin)
        AddInventoryEventFilter (None)

    endEvent

    ;--------------------------------------------------------------------
    ;
    Event OnItemRemoved (Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)

        if akDestContainer
            return
        endif

        ;  is still in the vault or is otherwise busy:
        If libs.PlayerIsBusy (IncludePowerArmor = true)
            Return
        EndIf

        float chance
        int cntkeys = kLastSafeTried.getitemcount (libs.DD_kw_IsDDKey)

        if cntkeys
            chance = Math.Max (Math.Min (Bo57_CL_KeyDropChance.GetValue() * cntkeys, 10.0), 2.0)
        endif

        chance += Bo57_CL_slaveitemsdropchance.GetValue()

        log ("cumulated chance " + chance)

        If triggerTrap (chance)
            bUseParasiteList = false
            bUseSafeList     = true

            UI.CloseMenu ("LockpickingMenu")

            startDeviousEffectSafe ()
        endif
    EndEvent

    ;------------------------------------------------------------
    ;
    Event OnMenuOpenCloseEvent (string asMenuName, bool abOpening)
        if !abOpening
            if asMenuName == "LockpickingMenu"
                GotoState ("")
            endif
        endif
    endEvent

endState


;------------------------------------------------------------------
;
;------------------------------------------------------------------

state UnlockedSafeOpened

    ;------------------------------------------------------------
    ;
    Event OnBeginState (string asOldState)
        Tools.convertDDkeys (true)
    endEvent

    ;------------------------------------------------------------
    ;
    Event OnEndState (string asOldState)
        log ("safe closed " + kLastSafeUsed)
        Tools.convertDDkeys (false)             ; only converts items in players inventory

        if kLastSafeUsed
            int remains = kLastSafeUsed.getitemcount (libs.DD_kw_IsDDKey)
            ; Log ("no of keys remaining in safe " + remains)

            if remains
                ; debug.Notification ("Now lets lock this safe")

                int isel = Bo57_Ask_LockSafe.show()   ; 0 = do noting, 1 = lock safe

                if isel == 1
                    if kLastSafeUsed.GetLockLevel() == 0
                        int ll = (Player.getvalue (lockpicking) / 26) as int
                        kLastSafeUsed.Setlocklevel (ll * 25)
                    endif
                    kLastSafeUsed.lock ()

                    int iId = kLastSafeUsed.GetFormID()
                    If !hasbeenlooted (iId)
                        rememberID (iId)    ; Block retrigger for a while
                    endif
                endif
            endIf

            if !kLastSafeUsed.haskeyword (WorkshopcontainerKeyword)
                if remains
                    Bo57_SafesforKeysRC.Addref (kLastSafeUsed)               ; created Workshopsafes do not respawn
                    Log ("no of safes total used " + Bo57_SafesforKeysRC.getcount())
                else
                    Bo57_SafesforKeysRC.Removeref (kLastSafeUsed)
                endif
            endif
            kLastSafeUsed = None
        endif
   endEvent

    ;------------------------------------------------------------
    ;
    Event OnMenuOpenCloseEvent (string asMenuName, bool abOpening)

        if !abOpening
            if asMenuName == "ContainerMenu"
                GotoState ("")
            endif
        endif
   endEvent
 endstate



