Scriptname Bo57:Bo57_MCMandToolsScript extends Quest

Group ActorsAndActorValues
  Actor             Property Player                      Auto Const
  Race              Property HumanRace                   Auto Const
  ActorValue        Property Paralyze                    Auto
endGroup

;---------------------------------------------------------------
Group MCMSettings
  GlobalVariable    Property Bo57_MCM_CL_FollowerItems   Auto
  GlobalVariable    Property Bo57_MCM_Use_Roggvir        Auto
endGroup

;---------------------------------------------------------------
Group MiscObjects
  Miscobject        Property Bo57_RestraintsKey          Auto Const
  Miscobject        Property Bo57_ChastityKey            Auto Const
  Miscobject        Property Bo57_SecurityKey            Auto Const
endGroup

;---------------------------------------------------------------
Group FormLists
  FormList          Property Bo57_kw_ItemMod_INR_List    Auto Const
  FormList          Property Bo57_DD_BreakList_Rendered  Auto Const
  FormList          Property Bo57_DD_BreakList_Inventory Auto Const
  FormList          Property Bo57_CL_rogglist            Auto Const
endGroup

;---------------------------------------------------------------
Group DDLibrary
  DD:DD_Library     Property libs                        Auto Const ; DD Base API
endGroup

;---------------------------------------------------------------
Group RealHandcuffs
  Bool              Property RHInstalled = false         Auto
  GlobalVariable    Property Bo57_RH_Detected            Auto
endGroup

;---------------------------------------------------------------
Group FO4NudeBasics
  Bool              Property FO4NBInstalled = false      Auto
  GlobalVariable    Property Bo57_F4b_GagQuestruns       Auto
  Quest             Property F4NBQuest                   Auto Hidden
  ScriptObject      Property F4NBPlayerRef               Auto Hidden
endGroup

;---------------------------------------------------------------
Group RoggDDItemsManager
  Bool              Property RoggDDMInstalled = false    Auto
  Quest             Property RoggDDQuest                 Auto Hidden
  FormList          Property RoggDDMChastityBelts        Auto Hidden
  ScriptObject      Property soRoggDDManager             Auto Hidden
endGroup

;---------------------------------------------------------------
Group SAKR
  Bool              Property SAKRInstalled = false       Auto
  FormList          Property SAKR_alltops                Auto Const
  FormList          Property SAKR_allbottoms             Auto Const
  FormList          Property SAKR_allskirts              Auto Const
  FormList          Property SAKR_alllegs                Auto Const
  FormList          Property SAKR_alllewd                Auto Const
endGroup

;---------------------------------------------------------------
Group SanityFramework
  Bool              Property SDSFInstalled = false       Auto
  ActorValue        Property SD_Sanity                   Auto Hidden
  ActorValue        Property SD_Depression               Auto Hidden
  ActorValue        Property SD_Grief                    Auto Hidden
  ActorValue        Property SD_Trauma                   Auto Hidden
  ActorValue        Property SD_Stress                   Auto Hidden
  ActorValue        Property SD_Alignment                Auto Hidden
  ActorValue        Property SD_Estrogen                 Auto Hidden
  ActorValue        Property SD_Testosterone             Auto Hidden
  ActorValue        Property SD_Age                      Auto Hidden
endGroup

;---------------------------------------------------------------
Group SexAttributes
  Bool              Property FPSAInstalled = false       Auto
  Quest             Property FPSAQuest                   Auto Hidden
  GlobalVariable    Property FPSAWear                    Auto Hidden
  GlobalVariable    Property FPSAWillpower               Auto Hidden
  GlobalVariable    Property FPSASelfesteem              Auto Hidden
  GlobalVariable    Property FPSASpirit                  Auto Hidden
  GlobalVariable    Property FPSAOrientation             Auto Hidden
  GlobalVariable    Property FPSASexReputation           Auto Hidden
  GlobalVariable    Property FPSATrauma                  Auto Hidden
  GlobalVariable    Property FPSAIntoxicationLevel       Auto Hidden
  GlobalVariable    Property FPSAArousal                 Auto Hidden
  Perk              Property FPSASlutPerk                Auto Hidden
  FPA:FPA_Main      Property FPSAMain                    Auto Hidden
endGroup

;---------------------------------------------------------------
Group ProvocativePerks
  Bool              Property PVPKInstalled = false       Auto
  Perk              Property PVPKNymphomania             Auto Hidden
endGroup

;---------------------------------------------------------------
Group ItemStorage
  FormList          Property Bo57_FoodKeywords          Auto Const Mandatory
  Message           Property Bo57_FoodBoxChoice         Auto Const Mandatory
  ObjectReference   Property Bo57_FoodLocker            Auto Const Mandatory
  MiscObject        Property Bo57_FoodBox               Auto Const Mandatory
  ReferenceAlias    Property Bo57_FoodBoxAlias          Auto Const Mandatory
  Bool              Property Bo57_FoodAskToDrop = true  Auto
endGroup

;---------------------------------------------------------------
Group Idles
  Idle              Property RaiderSheath                Auto Const
endGroup

;---------------------------------------------------------------
Group Globals
  GlobalVariable    Property Bo57_NudePantAndTop         Auto Const ; Nude must be both pants and top to count
endGroup

;---------------------------------------------------------------
Group Keywords
  Keyword           Property Bo57_kw_CursedKey           Auto Const Mandatory
  Keyword           Property cc_ma_Pipboy                Auto Const Mandatory ; [KYWD:0024A0B8]
endGroup

;---------------------------------------------------------------
Group StringArrays
  String[]          Property Bo57_kw_ItemMod_INR_Colors  Auto Const
endGroup


;---------------------------------------------------------------
; local variables
actor []          akPlayerFollowers
string            sModName

;===============================================================

; How to treat a trap for followers
;  0 = No handling
;  1 = always handle together with  player
;  2 = trap only followers, handle player if no follower is found or no item equipable

;---------------------------------------------------------------
;
;---------------------------------------------------------------

function Log (string s)
   Debug.Trace ("Bo57-Tools " + s)
Endfunction

;--------------------------------------------------------------------
;
Event Actor.OnPlayerLoadGame (Actor akSender)
   log ("OnPlayerLoadGame")
   initExternalMods ()
EndEvent

;--------------------------------------------------------------------

Event OnQuestInit ()
   RegisterForRemoteEvent (Player, "OnPlayerLoadGame");
   initExternalMods ()
EndEvent

; does some other initializations too
function initExternalMods()
  ; Roggvir's DD Items Manager
  ; https://www.loverslab.com/files/file/5259-roggvirs-dd-items-manager/
  sModName = "Rogg DD Items Manager.esp"
  RoggDDMInstalled = Game.IsPluginInstalled (sModName)
  If RoggDDMInstalled
    RoggDDQuest          = Game.GetFormFromFile(0x00004C50, sModName) as Quest
    RoggDDMChastityBelts = Game.getFormFromFile(0x00004C6B, sModName) as FormList
    soRoggDDManager      = RoggDDQuest.CastAs("Rogg:DDManager")
  Else
    soRoggDDManager = None
  endIf

  ; Real Handcuffs
  ; https://www.loverslab.com/files/file/8485-real-handcuffs/
  sModName = "RealHandcuffs.esp"
  RHInstalled = Game.IsPluginInstalled (sModName)
  if RHInstalled
    Bo57_RH_Detected.SetValueInt(1)
  else
    Bo57_RH_Detected.SetValueInt(0)
  endIf

  ; FO4 Nude Basics
  ; https://www.loverslab.com/files/file/22644-fo4-nude-basics/
  sModName = "FO4_Basics.esp"
  FO4NBInstalled = Game.IsPluginInstalled (sModName)
  If FO4NBInstalled
    F4NBQuest = Game.GetFormFromFile(0x00015D96, sModName) as Quest

    if F4NBQuest
      if F4NBQuest.isRunning()
        F4NBPlayerRef = F4NBQuest.GetAlias(0).CastAs("F4B:F4B_main_PlayerrefScript")
        Bo57_F4b_GagQuestruns.SetValueInt(1)

        RegisterForRemoteEvent (F4NBQuest, "OnQuestShutDown")
      else
        F4NBPlayerRef = none
        Bo57_F4b_GagQuestruns.SetValueInt(0)

        RegisterForRemoteEvent (F4NBQuest, "OnQuestInit")
      endIf
    endIf
  else
    F4NBPlayerRef = none
    Bo57_F4b_GagQuestruns.SetValueInt(0)
  endIf

  ; Sanity Framework
  ; https://www.loverslab.com/files/file/18664-sanity-framework/
  sModName = "SD_MainFramework.esp"
  SDSFInstalled = Game.IsPluginInstalled (sModName)
  If SDSFInstalled
    SD_Sanity     = Game.getFormFromFile( 0x00001733, sModName) as ActorValue
    SD_Depression = Game.getFormFromFile( 0x0001B17D, sModName) as ActorValue
    SD_Grief      = Game.getFormFromFile( 0x0001B17E, sModName) as ActorValue
    SD_Trauma     = Game.getFormFromFile( 0x0001B17F, sModName) as ActorValue
    SD_Stress     = Game.getFormFromFile( 0x00001ECF, sModName) as ActorValue
    ; SD_Alignment    "Alignment"    [AVIF:06002ECF]
    ; SD_Estrogen     "Estrogen"     [AVIF:0601F6A3]
    ; SD_Testosterone "Testosterone" [AVIF:0601F6A2]
    ; SD_Age          "Age"          [AVIF:0601F6A4]
  endIf
  
  ; [AAF] Sex Attributes - Framework
  ; https://www.loverslab.com/files/file/5436-aaf-sex-attributes-framework-4172023/
  sModName = "FPAttributes.esp"
  FPSAInstalled = Game.IsPluginInstalled (sModName)
  If FPSAInstalled
    FPSAQuest             = Game.GetFormFromFile(0x00000F99, "FPAttributes.esp") as Quest
    FPSAMain              = FPSAQuest as FPA:FPA_Main
    FPSAWear              = Game.GetFormFromFile(0x00000FAA, sModName) as GlobalVariable
    FPSAWillpower         = Game.GetFormFromFile(0x00000FAB, sModName) as GlobalVariable
    FPSASelfesteem        = Game.GetFormFromFile(0x00000FAC, sModName) as GlobalVariable
    FPSASpirit            = Game.GetFormFromFile(0x00007A67, sModName) as GlobalVariable
    FPSAOrientation       = Game.GetFormFromFile(0x00000FAD, sModName) as GlobalVariable
    FPSASexReputation     = Game.GetFormFromFile(0x0001944F, sModName) as GlobalVariable
    FPSATrauma            = Game.GetFormFromFile(0x0001E80B, sModName) as GlobalVariable
    FPSAIntoxicationLevel = Game.GetFormFromFile(0x0001E80C, sModName) as GlobalVariable
    FPSAArousal           = Game.GetFormFromFile(0x0001E80D, sModName) as GlobalVariable
    FPSASlutPerk          = Game.GetFormFromFile(0x00019452, sModName) as Perk
  endIf

  ; Provocative Perks
  ; https://www.loverslab.com/files/file/24462-provocative-perks/
  sModName = "ProvocativePerks.esp"
  PVPKInstalled = Game.IsPluginInstalled (sModName)
  If PVPKInstalled
    PVPKNymphomania = Game.GetFormFromFile(0x0000086D, "ProvocativePerks.esp") as Perk
  endIf

  ; Skimpy Armor Keyword Resource
  ; https://www.loverslab.com/files/file/26435-skimpy-armor-keyword-resource-4132023/
  sModName = "SkimpyArmorKeywordResource.esm"
  SAKRInstalled = Game.IsPluginInstalled (sModName)
  If SAKRInstalled
    ; sakr_kwd_armorTopFull       00000802
    ; sakr_kwd_armorTopBreast     00000803
    ; sakr_kwd_armorTopLewd       00000804
    ; sakr_kwd_braNormal          0000080B
    ; sakr_kwd_braBikini          0000080C
    ; sakr_kwd_braMicroBikini     0000080D
    ; sakr_kwd_braTagSheer        000026AF
    ; sakr_kwd_topFull            0000081A
    ; sakr_kwd_topCleavage        0000081B
    ; sakr_kwd_topLowCutCleavage  0000081C
    ; sakr_kwd_topTagCropTop      0000081D
    ; sakr_kwd_topTagTankTop      0000081E
    ; sakr_kwd_topTagHalterTop    0000081F
    ; sakr_kwd_topTagSideBoob     00000820
    ; sakr_kwd_topTagTight        00000821
    ; sakr_kwd_topTagSheer        00000822
    SAKR_alltops.Revert()
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x00000802, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x00000803, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x00000804, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000080B, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000080C, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000080D, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x000026AF, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000081A, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000081B, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000081C, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000081D, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000081E, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x0000081F, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x00000820, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x00000821, sModName) as Keyword )
    SAKR_alltops.AddForm( Game.getFormFromFile( 0x00000822, sModName) as Keyword )

    ; sakr_kwd_armorBottomFull    00000805
    ; sakr_kwd_armorBottomButt    00000806
    ; sakr_kwd_armorBottomLewd    00000807
    ; sakr_kwd_pantyNormal        00000808
    ; sakr_kwd_pantyThong         00000809
    ; sakr_kwd_pantyGString       0000080A
    ; sakr_kwd_pantyTagSheer      000026B0
    ; sakr_kwd_pantsLong          0000080E
    ; sakr_kwd_pantsShorts        0000080F
    ; sakr_kwd_pantsHotPants      00000810
    ; sakr_kwd_pantsThong         00000811
    ; sakr_kwd_pantsTagTight      00000812
    ; sakr_kwd_pantsTagSheer      00000813
    SAKR_allbottoms.Revert()
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000805, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000806, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000807, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000808, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000809, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x0000080A, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x000026B0, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x0000080E, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x0000080F, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000810, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000811, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000812, sModName) as Keyword )
    SAKR_allbottoms.AddForm( Game.getFormFromFile( 0x00000813, sModName) as Keyword )

    ; sakr_kwd_skirtLong          00000814
    ; sakr_kwd_skirtShort         00000815
    ; sakr_kwd_skirtMini          00000816
    ; sakr_kwd_skirtMicro         00000817
    ; sakr_kwd_skirtTagTight      00000818
    ; sakr_kwd_skirtTagSheer      00000819
    SAKR_allskirts.Revert()
    SAKR_allskirts.AddForm( Game.getFormFromFile( 0x00000814, sModName) as Keyword )
    SAKR_allskirts.AddForm( Game.getFormFromFile( 0x00000815, sModName) as Keyword )
    SAKR_allskirts.AddForm( Game.getFormFromFile( 0x00000816, sModName) as Keyword )
    SAKR_allskirts.AddForm( Game.getFormFromFile( 0x00000817, sModName) as Keyword )
    SAKR_allskirts.AddForm( Game.getFormFromFile( 0x00000818, sModName) as Keyword )
    SAKR_allskirts.AddForm( Game.getFormFromFile( 0x00000819, sModName) as Keyword )

    ; sakr_kwd_stockingsLong      000026B1
    ; sakr_kwd_stockingsTagShiny  000026B2
    ; sakr_kwd_stockingsTagSheer  000026B3
    ; sakr_kwd_shoesHighHeels     000026B4
    ; sakr_kwd_shoesKillerHeels   000026B5
    SAKR_alllegs.Revert()
    SAKR_alllegs.AddForm( Game.getFormFromFile( 0x000026B1, sModName) as Keyword )
    SAKR_alllegs.AddForm( Game.getFormFromFile( 0x000026B2, sModName) as Keyword )
    SAKR_alllegs.AddForm( Game.getFormFromFile( 0x000026B3, sModName) as Keyword )
    SAKR_alllegs.AddForm( Game.getFormFromFile( 0x000026B4, sModName) as Keyword )
    SAKR_alllegs.AddForm( Game.getFormFromFile( 0x000026B5, sModName) as Keyword )

    ; sakr_kwd_armorTopLewd       00000804
    ; sakr_kwd_pantyGString       0000080A
    ; sakr_kwd_braMicroBikini     0000080D
    ; sakr_kwd_skirtMicro         00000817
    SAKR_alllewd.Revert()
    SAKR_alllewd.AddForm( Game.getFormFromFile( 0x00000804, sModName) as Keyword )
    SAKR_alllewd.AddForm( Game.getFormFromFile( 0x0000080A, sModName) as Keyword )
    SAKR_alllewd.AddForm( Game.getFormFromFile( 0x0000080D, sModName) as Keyword )
    SAKR_alllewd.AddForm( Game.getFormFromFile( 0x00000817, sModName) as Keyword )

    log ("Skimpy Armor Keyword Resource is installed")
  endIf


  ; sModName = "AAF.esm"
  ; If bAFFInstalled
  ;   AAF:AAF_API AAF_API = Game.GetFormFromFile(0x00000F99, sModName) as AAF:AAF_API
  ;
  ;   AAF_protectedKeywords = Utility.VarToVarArray(AAF_API.GetProtectedEquipmentKeywords()) as Keyword[]
  ;
  ;   log ("Advanced Animation Framework is installed")
  ; endIf

endfunction

;---------------------------------------------------------------
; only returns true if a list with at least one item is created
;---------------------------------------------------------------

bool function RoggvirGetDDList ()
  If Bo57_MCM_Use_Roggvir.getvalueint() && soRoggDDManager

    Bo57_CL_rogglist.Revert()
    Var roggFormVar = soRoggDDManager.CallFunction ("PickRandomItemsRemote", new Var[0])
    log ("roggformvar " + roggFormVar)

    If roggFormVar
      form[] roggForms = Utility.VarToVarArray(roggFormVar) as form[]
      Int iLen = roggForms.Length
      if ilen
        While iLen
          ilen -= 1
          Bo57_CL_rogglist.AddForm( roggForms [iLen] )
          ; log ("Added to list " + roggForms [iLen].getname())
        EndWhile

        Return true
      endIf
    endIf
  endIf

  Return false
endfunction

;---------------------------------------------------------------
; future use ??
;---------------------------------------------------------------

Function MCM_StartStopQuest ()


EndFunction

;---------------------------------------------------------------
; Identify valid armor
;---------------------------------------------------------------
bool function isValidItem ( armor attire, bool checkDD = true )
    Return attire && attire.GetName() && attire.GetSlotMask() && !attire.haskeyword(cc_ma_Pipboy) && (!checkDD || (checkDD && !attire.haskeyword(libs.DD_kw_RenderedItem) && !attire.haskeyword(libs.DD_kw_InventoryItem)))
endfunction

;---------------------------------------------------------------
; Is Heavily Restrained
;---------------------------------------------------------------
bool function isHeavyRestrained( Actor akActor  )
   Return akActor.WornHasKeyword(libs.DD_kw_ItemType_WristCuffs) || \
          akActor.WornHasKeyword(libs.DD_kw_ItemType_LegCuffs) || \
          akActor.WornHasKeyword(libs.DD_kw_ItemType_Suit) || \
          akActor.wornhaskeyword (libs.DD_kw_EffectType_BondageMittens)
endfunction;

;---------------------------------------------------------------
; Is Naked
;---------------------------------------------------------------
bool function isNaked (  Actor akActor  )
    bool isNaked    = false
    bool hasTop     = false
    bool hasBottom  = false
    bool mustBeBoth = (Bo57_NudePantAndTop.GetValueInt() == 1)

    if SAKRInstalled
        Int idx = 3
        Int end = 43 const

        while (idx < end)
            Actor:WornItem wornValue = akActor.GetWornItem (idx)
            armor attire = wornValue.item as armor

            if isValidItem ( attire )
                if !hasTop && attire.HasKeywordInFormList( SAKR_alltops ) && !attire.HasKeywordInFormList( SAKR_alllewd )
                  hasTop = true;
                endIf
                if !hasBottom && (attire.HasKeywordInFormList( SAKR_allskirts ) || attire.HasKeywordInFormList( SAKR_allbottoms )) && !attire.HasKeywordInFormList( SAKR_alllewd )
                  hasBottom = true;
                endIf
            endIf

            idx += 1
        EndWhile
        
        isNaked = (mustBeBoth && !hasTop && !hasBottom) || (!mustBeBoth && (!hasTop || !hasBottom))
    else
        Actor:WornItem wornValue = akActor.GetWornItem (3)
        armor attire = wornValue.item as armor

        isNaked = !isValidItem ( attire )
    endIf

    Return isNaked
endfunction

;---------------------------------------------------------------
; Strip Actor
;---------------------------------------------------------------
function stripActor( Actor akActor )
    if SAKRInstalled && !FO4NBInstalled
        Int idx        = 3
        Int end        = 43 const

        while (idx < end)
            Actor:WornItem wornValue = Player.GetWornItem (idx)
            armor attire = wornValue.item as armor

            if isValidItem( attire ) && (attire.HasKeywordInFormList( SAKR_alltops ) || attire.HasKeywordInFormList( SAKR_allskirts ) || attire.HasKeywordInFormList( SAKR_allbottoms ))
                unequipTryBreaking( akActor, attire )
            endIf

            idx += 1
        EndWhile
    else
        Actor:WornItem wornValue = akActor.GetWornItem (3)
        armor attire = wornValue.item as armor

        if isValidItem( attire )
            unequipTryBreaking( akActor, attire )
        endIf
    endIf
endfunction

;---------------------------------------------------------------
; F4NB/SAKR Armor Degradiation or Unequip
;---------------------------------------------------------------
function unequipTryBreaking (Actor akActor, Armor akArmor)
    if !isValidItem( akArmor )
        Return
    endIf

    ; Conflicts with SAKR keyword stripping
    ; F4NB rips players clothing
    if !SAKRInstalled && F4NBPlayerRef
        Var[] Args = New Var[0]
        F4NBPlayerRef.CallFunction ("Do_bodyarmor_break", Args)

        Return
    endIf

    akActor.unequipitem ( akArmor, false, true )
endFunction

;---------------------------------------------------------------
; Fix Actor Weapon idle
;---------------------------------------------------------------
Bool function isWeaponIdleFixed ( Actor akActor )
    if akActor.isWeaponDrawn()

        akActor.playidle (RaiderSheath)
        Utility.wait (1.0)
        Return false
     else
        Return true
    endIf
endfunction

;---------------------------------------------------------------
; Identify possible followers
;---------------------------------------------------------------
function identifyPossibleFollowers (explosion knockdown)

  ; I do not know if the common solution with a search quest is better
  actor [] followers = Game.GetPlayerFollowers( )
  actor    companion
  Float    DistanceToPlayer

  akPlayerFollowers = new actor [0]

  Int   i = followers.length
  while i
    i -=1
    companion = followers[i]

    if companion && ( companion.getrace () == Humanrace )
      ; for now ignore ghouls, dogs, robots etc..
      ; Log ("follower found " + companion.getdisplayname ())

      ; ignore followers in power armor or far away
      ; Two should be enough - my opinion
      if !companion.IsInPowerArmor() && Player.GetDistance(companion) <= 1000.0 && akPlayerFollowers.length < 2
        Log ("follower added " + companion.getdisplayname ())
        akPlayerFollowers.add (companion)
        companion.placeatme (knockdown, 1)
      endIf
    endIf
  endwhile
endfunction


;---------------------------------------------------------------
; possibly paralyze followers
;---------------------------------------------------------------
function paralyzeFollowers (Int iset = 0)
   Int   i = akPlayerFollowers.length
   while i
      i -=1

      akPlayerFollowers[i].SetValue (Paralyze, iset)
      ; Log ("Paralyze" + akPlayerFollowers[i].getdisplayname ())
   endwhile
endfunction


;---------------------------------------------------------------
; adopted from Devious devices original without checking dependencies
;---------------------------------------------------------------
Bool Function EquipRandomDevice(Actor akActor, FormList ItemList)
  Armor Device
  FormList Branch = ItemList
  Form Frm
  Int i = 1

  While i > 0
    i -= 1
    Frm = Branch.GetAt(Utility.RandomInt(0, Branch.GetSize() - 1))
    If (Frm As Armor)
      Device = Frm As Armor
      ; This item is an actual DD device, so let's equip it, if we can and reset the tree if we can't!
      ; Bool Function EquipDevice(Actor akActor, Armor InventoryDevice, Armor RenderedDevice = none, Bool CheckDependencies = True, Keyword Pose = None, String Color = "none")

      If !akActor.WornHasKeyword(libs.GetDeviceKeyword(Device)) && libs.EquipDevice(akActor, Device, None, False, None, None)
        Return True
      endIf

    ElseIf (Frm As FormList)
      ; branch down if it's a form list
      Branch = Frm As FormList
      i += 1
    endIf
  EndWhile

  Return False
EndFunction

;---------------------------------------------------------------
;
;---------------------------------------------------------------
Int function TryToAddMoreItems (float base)

   Int iadd
   base *= 0.5
   if Utility.randomfloat () <= base
      iadd += 1
   endIf
   if Utility.randomfloat () <= (base * 0.5)
      iadd += 1
   endIf

   Return iAdd

Endfunction

;---------------------------------------------------------------
; possibly equip items on single followers
;---------------------------------------------------------------
Int function handleCompanionItems (actor companion, formlist equipfrom_list, bool bremkeys)

   Int   eq_cnt
   actorbase cabase = companion.getactorbase ()

   if cabase.getsex () < 1
      ; log ("male companion ignored for now")

      ; We do not handle males -> always use player to equip someting lateron
      Return 0
   endIf

   companion.disable ()
   Utility.Wait (0.1)

   Int i = 1 + TryToAddMoreItems (0.9)
   while i > 0
      i -= 1;
      If EquipRandomDevice(companion, equipfrom_list)
         eq_cnt = 1
      else
         i = 0;          Terminate without further trials to equip something
      endIf
   Endwhile

   if bremkeys
      companion.RemoveItem (libs.DD_RestraintsKey, -1)
      companion.RemoveItem (libs.DD_ChastityKey, -1)
   endIf

   companion.enable ()

   Return eq_cnt

endfunction


;---------------------------------------------------------------
; possibly equip items on followers
;---------------------------------------------------------------
bool function equipFollowerItems (formlist equipfrom_list, bool bremkeys)

; How to treat a trap for followers
;  0 = No handling
;  1 = always handle together with  player
;  2 = trap only followers, handle player if no follower is found or no item equipable

   Int imode = Bo57_MCM_CL_FollowerItems.getvalueint ()
   bool trap_player = true

   if imode

      actor    companion
      Int      eq_cnt

      Int   f = akPlayerFollowers.length
      while f
         f -=1
         companion = akPlayerFollowers[f]

         eq_cnt += handleCompanionItems (companion, equipfrom_list, bremkeys)
       endwhile

      ; log ("followeritems mode " + imode + " eqcnt " + eq_cnt)
      if imode == 2
         if eq_cnt
            trap_player = false
         endIf
      endIf
   endIf

   Return trap_player   ; trap the player

endfunction

;---------------------------------------------------------------
; Some Helpers to replace items in a player container
;---------------------------------------------------------------
Int function replaceitems (form toreplace, form withitem, bool bsilent=true)

   Int iCnt
   if toreplace && withitem                           ; There seems to be an initialization problem sometimes
      iCnt = Player.GetItemCount (toreplace)

      if icnt
         Player.RemoveItem (toreplace, icnt, true)    ; silent
         Player.addItem (withitem, icnt, bsilent)
      endIf
      log ("converted " + toreplace.getname() + " icnt " + icnt)
   endIf

   Return icnt
endfunction

;------------------------------------------------------------------
; convert DD keys to misc items and vice versa
;------------------------------------------------------------------
function convertDDkeys (Bool btoMiscitem, bool bsilent=true)

   if btoMiscitem
      replaceitems (libs.DD_RestraintsKey,   Bo57_RestraintsKey)
      replaceitems (libs.DD_ChastityKey,     Bo57_ChastityKey)
      replaceitems (libs.DD_HighSecurityKey, Bo57_SecurityKey)
   else
      if Player.GetItemCount (Bo57_kw_CursedKey)
         replaceitems (Bo57_RestraintsKey,      libs.DD_RestraintsKey,   bsilent )
         replaceitems (Bo57_ChastityKey,        libs.DD_ChastityKey,     bsilent )
         replaceitems (Bo57_SecurityKey,        libs.DD_HighSecurityKey, bsilent )
      endIf
   endIf

endfunction

;---------------------------------------------------------------
; Handle OnQuest events for mods
;---------------------------------------------------------------
Event Quest.OnQuestInit (Quest kQuest)
    if kQuest == F4NBQuest
        F4NBPlayerRef = F4NBQuest.GetAlias(0).CastAs("F4B:F4B_main_PlayerrefScript")
        Bo57_F4b_GagQuestruns.SetValueInt (1)

        RegisterForRemoteEvent(kQuest, "OnQuestShutdown")
        UnregisterForRemoteEvent(kQuest, "OnQuestInit")
    endIf
EndEvent

Event Quest.OnQuestShutdown (Quest kQuest)
    if kQuest == F4NBQuest
        F4NBPlayerRef = none
        Bo57_F4b_GagQuestruns.SetValueInt (0)

        RegisterForRemoteEvent(kQuest, "OnQuestInit")
        UnregisterForRemoteEvent(kQuest, "OnQuestShutdown")
    endIf
EndEvent

;*******************************************************************************
;
; Bridge functions for my GTS mod

;---------------------------------------------------------------
; get equipped devious armor inventory objects
;---------------------------------------------------------------
; Result  var []
; Casting to a struct is not supported, thats why we must store interesting armor
; in an array and use this function as an interface to the DD Library

var[] Function DataBaseRetrieve (Actor akActor, Bool Getkeys)

;/       Actor slave = None
0        Armor nipplepiercings = None
1        Armor pluganal = None
2        Armor plugvaginal = None
3        Armor blindfold = None           restraintskey
4        Armor legcuffs = None
5        Armor gloves = None
6        Armor collar = None
7        Armor gag = None
8        Armor hood = None
9        Armor harness = None
10       Armor bra = None
11       Armor belt = None
12       Armor suit = None
13       Armor wristcuffs = None  /;

   var [] Result = New Var[2]             ; both  initialized with None
   DD:DD_Library:WornDevices wds = Libs.databaseRetrieve (akActor)
   if wds
      armor [] worn = New  armor [14]
      worn [0] = wds.nipplepiercings
      worn [1] = wds.pluganal
      worn [2] = wds.plugvaginal
      worn [3] = wds.blindfold
      worn [4] = wds.legcuffs
      worn [5] = wds.gloves
      worn [6] = wds.collar
      worn [7] = wds.gag
      worn [8] = wds.hood
      worn [9] = wds.harness
      worn [10] = wds.bra
      worn [11] = wds.belt
      worn [12] = wds.suit
      worn [13] = wds.wristcuffs
      Result [0] = Utility.VarArrayToVar (worn as Var[])

      if Getkeys
         key [] keylist = New key [14]
         Int i
         while i < 14
            if worn [i]
               keylist [i] = libs.GetDeviceKey (worn [i])
            endIf
            i += 1
         endwhile

         Result [1] = Utility.VarArrayToVar (keylist as Var[])
      endIf

      ;/ For testing
      armor x
      Int j = libs.DD_PlayerEquippedItems.getsize ()
      log ("dd worn formlist len = " + j)
      while j
         j -= 1
         x = libs.DD_PlayerEquippedItems.getat (j) as armor
         log ("DD_PlayerEquippedItems " + x)
      endwhile
      /;
   endIf

   Return result

endfunction

; ----------------------------------------------------------------------------------------
;
; ----------------------------------------------------------------------------------------

var[] Function DD_Slot_Getcolor ( Actor akActor, Int islot, armor attire)

   string       scolor
   ObjectMod    Omod
   ObjectMod [] objectMods = akActor.GetWornItemMods(islot)
   Int          ip

; log ("Actor " + akActor + " slot " + islot + " armor " + attire)
; log ("objectmods " + objectMods)

   If objectMods
      Int im
      ObjectMod:PropertyModifier[] props
      Keyword kw

      While im < objectMods.Length
         Omod  = objectMods [im]

         props = OMod.GetPropertyModifiers()
			While (ip < props.length)
				If props[ip].target == Omod.Armor_Target_pkKeywords

               kw = props[ip].object as keyword
               if kw
                  Int iColidx = Bo57_kw_ItemMod_INR_List.find (kw)
                  if iColidx >= 0
                     ; We found some of the color keywords used by devious devices

                     sColor = Bo57_kw_ItemMod_INR_Colors [iColidx]

                     ip = props.length       ; Now terminate inner loop
                     im = objectMods.Length  ; and exterior loop
                  endIf
               endIf
				endIf
				ip += 1
			EndWhile

         im += 1
      EndWhile

   endIf

   If scolor
      ; This formlists are not complete and are intended for the bread DD system in Nude Basics
      ; So not all colors are found

      ip = Bo57_DD_BreakList_Rendered.find (attire)
      if ip >= 0
         attire = Bo57_DD_BreakList_Inventory.getat (ip) as armor
         var [] Result = New Var[2]
         Result [0] = sColor
         Result [1] = attire

         Return Result
      endIf
   endIf

   Return None             ; Nothing found

endfunction


; ----------------------------------------------------------------------------------------
; TEST
; ----------------------------------------------------------------------------------------
; Try to reset the scripts, so that local variables on the script will go to initial value

;/
Bool Function EquipDevice(Actor akActor, Armor InventoryDevice, Armor RenderedDevice, Bool CheckDependencies, Keyword Pose, String Color)
	ObjectReference tmpORef = Player.placeAtMe(InventoryDevice, abInitiallyDisabled = true)
	DD:DD_RestraintScript tmpZRef = tmpORef as DD:DD_RestraintScript
   tmpZref.Reset ()
   tmpORef.Delete()

   libs.EquipDevice (akActor, InventoryDevice, None, False, None)
   Return true
EndFunction
/;




